/* This file was generated by syncqt. */
#ifndef QT_QTQMLDEVTOOLS_VERSION_H
#define QT_QTQMLDEVTOOLS_VERSION_H

#define QTQMLDEVTOOLS_VERSION_STR "5.6.2"

#define QTQMLDEVTOOLS_VERSION 0x050602

#endif // QT_QTQMLDEVTOOLS_VERSION_H
