/* This file was generated by syncqt. */
#ifndef QT_QTWEBKITWIDGETS_VERSION_H
#define QT_QTWEBKITWIDGETS_VERSION_H

#define QTWEBKITWIDGETS_VERSION_STR "5.6.2"

#define QTWEBKITWIDGETS_VERSION 0x050602

#endif // QT_QTWEBKITWIDGETS_VERSION_H
