pre {
  background-color: #f0f0f0;
  font-family: Courier, monospace;
  font-weight: 600;
  vertical-align: top;
  margin: 15px 85px 15px 35px;
  padding: 25px;
  width: 90%;
  overflow-x:auto;
}

pre a[href] {
  color: #5caa15;
}

p {
  width: 70%;
  margin: 15px 0px 10px 15px;
}

table p {
  margin: 0px;
  padding: 0px;
}

a[href] {
  color: #007330;
  text-decoration: none;
}

/* Different color for ext. links */
a[href|="http://"], a[href|="https://"] {
    color: #6bb8db;
}

h1.title {
  margin-top: 30px;
  margin-left: 6px;
  padding: 6px;
}

h2, p.h2 {
  background-color: #F2F3F4;
  padding: 4px;
  margin: 30px 0px 20px 10px;
}

h3 {
  margin: 30px 0px 30px 6px;
}

ul, ol {
  margin-top: 4px;
  margin-bottom: 0px;
}

ul li, ol li {
  margin-bottom: 8px;
}

.mainContent li.level2 {
  margin-left: 16px;
}

.rightAlign {
  text-align: right;
}

h3.fn, span.fn {
  border-width: 3px;
  border-style: solid;
  border-color: #aaaaaa;
  background-color: #eeeeee;
  word-spacing: 3px;
  padding: 5px;
  text-decoration: none;
  font-weight: 400;
  margin: 45px 0px 0px 6px;
}

table {
  max-width: 80%;
  padding: 15px 45px 15px 15px;
}

table th {
  text-align: left;
  padding: 8px;
}

table td {
  padding: 6px 10px 6px 10px;
}

table tr.odd {
  background-color: #eeeeee;
}

table.qmlname td {
  padding: 0px;
  margin-left: 6px;
}

table.qmlname p .name,
h3.fn .name, h3.fn .type {
  font-weight: bold;
}

.context h3.fn {
  font-weight: 400;
}

.qmlreadonly, .qmldefault {
  font-family: Courier, monospace;
  margin-right: 6px;
}

code {
  font-family: Courier, monospace;
  font-weight: 400;
}

p.naviNextPrevious {
  text-align: right;
  margin-right: 20px;
}

span.naviSeparator {
  white-space: pre;
}

.toc h3 {
  margin: 0px 0px 10px 6px;
}

.toc ul {
  list-style-type: none;
}

.navigationbar table {
  padding: 0;
  margin: 0;
}

.navigationbar table tr {
  background-color: #eeeeee;
}

td#buildversion {
  background-color: #ffffff;
}

.footer, .footer p {
  padding: 5px 0px 5px 0px;
  margin: 45px 15px 5px 15px;
  font-size: 8.5pt;
  background-color: #cccccc;
}

.footer p {
  margin: 0px;
}
