/****************************************************************************
**
** Copyright (C) 2015 The Qt Company Ltd.
** Contact: http://www.qt.io/licensing/
**
** This file is part of the QtWidgets module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL21$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see http://www.qt.io/terms-conditions. For further
** information use the contact form at http://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 or version 3 as published by the Free
** Software Foundation and appearing in the file LICENSE.LGPLv21 and
** LICENSE.LGPLv3 included in the packaging of this file. Please review the
** following information to ensure the GNU Lesser General Public License
** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** As a special exception, The Qt Company gives you certain additional
** rights. These rights are described in The Qt Company LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** $QT_END_LICENSE$
**
****************************************************************************/

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//


#define IDR_MAIN_MENU       102
#define IDR_MAIN_MENU2      103
#define IDR_MAIN_MENU3      104
#define IDS_EXIT            105
#define IDS_MENU            106
#define IDS_LEFTMENU        107
#define IDM_ABOUT           108
#define IDM_VIEW            109
#define IDM_ITEM1           108
#define IDM_ITEM2           109
#define IDM_ITEM3           110
#define IDM_ITEM4           111
#define IDM_ITEM5           112
#define IDM_ITEM6           113
#define IDM_ITEM7           114
#define IDM_ITEM8           115
#define IDS_MENU1           116
#define IDS_MENU2           117
#define IDS_MENU3           118
#define IDS_MENU4           119
#define IDS_MENU5           120
#define IDS_MENU6           121
#define IDS_MENU7           122
#define IDS_MENU8           123
#define IDR_MAIN_MENU4      124
#define IDR_MAIN_MENU5      125
#define IDM_EXIT            40000
#define IDM_MENU            40001
#define IDM_LEFTMENU        40002
#define IDM_MENU1           40003
#define IDM_MENU2           40004
#define IDM_MENU3           40005
#define IDM_MENU4           40006
#define IDM_MENU5           40007
#define IDM_MENU6           40008
#define IDM_MENU7           40009
#define IDM_MENU8           40010

