/* This file was generated by syncqt. */
#ifndef QT_QTSCRIPT_VERSION_H
#define QT_QTSCRIPT_VERSION_H

#define QTSCRIPT_VERSION_STR "5.6.2"

#define QTSCRIPT_VERSION 0x050602

#endif // QT_QTSCRIPT_VERSION_H
