/****************************************************************************
**
** Copyright (C) 2015 The Qt Company Ltd.
** Contact: http://www.qt.io/licensing/
**
** This file is part of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL21$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see http://www.qt.io/terms-conditions. For further
** information use the contact form at http://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 or version 3 as published by the Free
** Software Foundation and appearing in the file LICENSE.LGPLv21 and
** LICENSE.LGPLv3 included in the packaging of this file. Please review the
** following information to ensure the GNU Lesser General Public License
** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** As a special exception, The Qt Company gives you certain additional
** rights. These rights are described in The Qt Company LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef QCAMERAINFO_H
#define QCAMERAINFO_H

#include <QtMultimedia/qcamera.h>
#include <QtCore/qsharedpointer.h>

QT_BEGIN_NAMESPACE

class QCameraInfoPrivate;

class Q_MULTIMEDIA_EXPORT QCameraInfo
{
public:
    explicit QCameraInfo(const QByteArray &name = QByteArray());
    explicit QCameraInfo(const QCamera &camera);
    QCameraInfo(const QCameraInfo& other);
    ~QCameraInfo();

    QCameraInfo& operator=(const QCameraInfo& other);
    bool operator==(const QCameraInfo &other) const;
    inline bool operator!=(const QCameraInfo &other) const;

    bool isNull() const;

    QString deviceName() const;
    QString description() const;
    QCamera::Position position() const;
    int orientation() const;

    static QCameraInfo defaultCamera();
    static QList<QCameraInfo> availableCameras(QCamera::Position position = QCamera::UnspecifiedPosition);

private:
    QSharedPointer<QCameraInfoPrivate> d;
};

bool QCameraInfo::operator!=(const QCameraInfo &other) const { return !operator==(other); }

#ifndef QT_NO_DEBUG_STREAM
Q_MULTIMEDIA_EXPORT QDebug operator<<(QDebug, const QCameraInfo&);
#endif

QT_END_NAMESPACE

#endif // QCAMERAINFO_H
